/* rmgcomm.c
 *
 * Redundancy manager (RMG) sockets communication handler
 *
 * This module provides the communication interface to the mate node for the rest
 * of the RMG process.
 *
 * This module is composed of a receiver thread, whose job is to continuously read
 * from the socket and post any received messages to the controller FSM module, and
 * utility routines for initializing the interface, sending packets, and closing
 * down the interface.
 *
 * Connectionless datagram (UDP) sockets are employed.  All socket I/O is blocking
 * (thus the use of a separate thread for receiving).  The cleanup routine closes
 * down the scoket, which also has the effect of killing the receiver thread.
 *
 * The sending module is responsible for timing for responses and issuing any desired
 * retransmissions.
 */


#ifdef WIN32
#include <windows.h>
/*#include <winsock.h>*/
#include <process.h>
#endif

#include <string.h>
#include <stdio.h>

#include "osdep.h"
#include "ctadef.h"
#include "ctaerr.h"
#include "nmstypes.h"
#include "ss7type.h"
#include "isupdef.h"
#include "demodef.h"


/* Constants and defaults */
#define REQUIRED_WINSOCK_VERSION    0x0101  /* requires at least winsock version 1.1 */

/* Globals */
static TSockAddrIn           remSock;    /* IP address/port for mate application */
static TSockAddrIn           locSock;    /* this application's socket */
static TSocket               Socket;     /* socket to receive on */

extern U16      locPort;
extern U16      remPort;
extern U32      remAddr;
extern Circuit *circuits;
extern short    traceFlags;
extern CTAHD    isupHndl;
extern int      standAlone;

void TraceCheckPoint(ChkPntMsg *chkPntMsg, int dirFlag)
{
    char       *callState;
    char       *transient;
    char       *blkState;
    char       *dir;

    switch (chkPntMsg->callState)
    {
        case CSIDLE:
            callState = "Idle    ";
            break;

        case CSINCBUSY:
            callState = "In Busy ";
            break;

        case CSOUTBUSY:
            callState = "Out Busy";
            break;

        default:
            callState = "UNKNOWN!";
            break;
    }
    switch (chkPntMsg->blkState)
    {
        case BSNOTBLK:
            blkState = "Not Blocked    ";
            break;

        case BSLOCBLK:
            blkState = "Loc Blocked    ";
            break;

        case BSRMTBLK:
            blkState = "Rmt Blocked    ";
            break;

        case BSLOCRMTBLK:
            blkState = "Loc/Rmt Blocked";
            break;

        default:
            blkState = "UNKNOWN!       ";
            break;
    }
    switch (dirFlag)
    {
        case DIR_TX:
            dir = "TX";
            break;

        case DIR_RX:
            dir = "RX";
            break;

        default:
            dir = "??";
            break;
    }
    if (chkPntMsg->transient)
        transient = "Transient";
    else
        transient = "         ";

    fprintf(stderr, "CheckPoint %s: %d %d %s %s %s\n", dir, chkPntMsg->cirId, chkPntMsg->mateId, blkState, callState, transient);
}


/*
 * UDPThread
 *
 * This is the entry point to the receiver thread; its job is to loop continuously
 * doing blocking reads on the socket and posting any received packets to the SCTL
 * module in the form of "data indication" events, until a socket error is received
 * (indicating that the socket has been shut down and its time to stop).
 *
 * Parameters:
 *
 *  pParam      pointer to socket handle to use
 *
 * Returns:     Always 0
 *
 */
void UDPThread( void *dummy )
{
#ifdef WIN32
    WSADATA         wsadata;        /* sockets version data */
    int             wsaret;         /* sockets return code */

    wsaret = WSAStartup(REQUIRED_WINSOCK_VERSION, &wsadata);
    if (wsaret != 0)
    {
        fprintf(stderr,  "UDPThread terminating, unable to initialize sockets API, error %d\n ", wsaret );
        THREAD_EXIT();
    }
    else if ((LOBYTE(wsadata.wVersion) != 1) || HIBYTE(wsadata.wVersion) != 1)
    {
        /* incompatible SOCKETS API version */
        fprintf(stderr, "UDPThread terminating, unsupported sockets API version (%d.%d)\n",
                 HIBYTE(wsadata.wVersion), LOBYTE(wsadata.wVersion));
        THREAD_EXIT();
    }
#endif

    /* next create a UDP datagram socket */
    if((SOCKET_OPEN( AF_INET, SOCK_DGRAM, 0, Socket )) == RET_FAILED )
    {
        /* fatal error */
        fprintf(stderr, "UDPThread terminating, unable to create socket, error %d", osGetLastError());
        THREAD_EXIT();
    }
    locSock.sin_port = locPort;
    locSock.sin_family = AF_INET;
    locSock.sin_addr.s_addr = htonl(INADDR_ANY);

    remSock.sin_port = remPort;
    remSock.sin_family = AF_INET;
    remSock.sin_addr.s_addr = remAddr;

    /* Bind the socket to our configured port */
    if(SOCKET_BIND(Socket, (TSockAddr*)&locSock, sizeof(locSock))
       != RET_SUCCESS)
    {
        fprintf(stderr, "UDPThread terminating, unable to bind socket, error %d", osGetLastError());
        THREAD_EXIT();
    }


    while (1)
    {
        CTA_EVENT           ctaEvent;
        TSockAddrIn         fromAddr;       /* IP address/port for mate application */
        Event              *evt;
        int                 fromLen;        /* length of socket address structure */
        int                 sret;           /* sockets return value */
        int                 recvLen;        /* length of received message */

        if ((evt = (Event *)calloc(1, sizeof(Event))) == NULL)
        {
            fprintf(stderr, "UDPThread terminating, memory allocation failure\n");
            THREAD_EXIT();
        }

        fromLen = sizeof(struct sockaddr_in);
        if ((SOCKET_RECVFROM(Socket, (char*) &evt->chkPntMsg, sizeof(ChkPntMsg), 0,
                             (TSockAddr*) &fromAddr, &fromLen, recvLen)) != RET_SUCCESS)
        {
            sret = osGetLastError();
            if( sret == ERR_MSGSIZE )
            {
                /* message too big, probably from some other application due to port
                 * number conflict - display warning so users can resolve conflict
                 */
                fprintf(stderr,  "UDPThread: Message from %s too long [len=%d], discarded", inet_ntoa(fromAddr.sin_addr), recvLen );
            }
#ifdef UNIX
            else if (sret == ECONNREFUSED)
            {
                /* ignore this error, which occurs before other ISUPdemo starts */
            }
#else
            else  if ((sret != WSAECONNREFUSED) &&
                      (sret != WSAECONNRESET) )
            {
                fprintf(stderr, "UDPThread terminating, socket receive failed %ld\n", osGetLastError());
                THREAD_EXIT();
            }
#endif
        }
        else if  (evt->chkPntMsg.msgId == CHKPNT_MSG)
        {
            if (traceFlags & CHKPNT_TRACE)
                TraceCheckPoint(&evt->chkPntMsg, DIR_RX);

            EventCircuit(0, evt->chkPntMsg.cirId, CHKPNT_EVT, evt);
        }
        else if  (evt->chkPntMsg.msgId == APPRDY_MSG)
        {
            memset(&ctaEvent, 0, sizeof(CTA_EVENT));

            ctaEvent.id     = APPREADY_EVT;
            ctaEvent.size   = 0;
            ctaEvent.buffer = NULL;
            ctaEvent.ctahd  = isupHndl;
            ctaQueueEvent(&ctaEvent);
            free(evt);
        }
        else
            fprintf(stderr, "UDPThread: Unknown message type 0x%x received\n", evt->chkPntMsg.msgId);
    }
}                               /* of RmgCommRecvThread */

void CheckPoint(Circuit *circuit)
{

    ChkPntMsg   chkPntMsg;         /* buffer for outgoing message */

    if( standAlone )
        return;                         /* nothing to do */

    /* construct message */

    chkPntMsg.msgId     = CHKPNT_MSG;
    chkPntMsg.cirId     = circuit->cirId;
    chkPntMsg.mateId    = circuit->mateId;
    chkPntMsg.callState = circuit->callState;
    chkPntMsg.transient = circuit->transient;
    chkPntMsg.blkState  = circuit->blkState;

    if (traceFlags & CHKPNT_TRACE)
      TraceCheckPoint(&chkPntMsg, DIR_TX);
    if (SOCKET_SENDTO(Socket, (char *) &chkPntMsg, sizeof(ChkPntMsg), 0,
                      (TSockAddr*) &remSock, sizeof(TSockAddrIn)) != RET_SUCCESS)

      {
	fprintf(stderr,  "UDP: Unable to send checkpoint message, error %ld\n", osGetLastError());
      }
}

void AppReady()
{

    ChkPntMsg   chkPntMsg;         /* buffer for outgoing message */

    if( standAlone )
        return;                         /* nothing to do */

    /* construct message */

    chkPntMsg.msgId     = APPRDY_MSG;

    if (SOCKET_SENDTO(Socket, (char *) &chkPntMsg, sizeof(ChkPntMsg), 0,
                      (TSockAddr*) &remSock, sizeof(TSockAddrIn)) != RET_SUCCESS)
      {
        fprintf(stderr,  "UDP: Unable to send ready message, error %ld\n", osGetLastError());
      }
}
