/*
 * term.c - sample call termination application
 *
 */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#ifdef WIN32
#include <conio.h>
#endif

#ifdef UNIX
#define _getch getchar
#endif

#ifdef UNIX
#include "sol-dep.h"
#endif /* UNIX */

#include "ss7type.h"
#include "isupdef.h"

/* globals */

U8 boardNum = 1;
CirId circuit = 1;
U16 switchType = ST_ANS92;

static char *circuitStates[] =
{
    "transient",
    "error",
    "error",
    "unequipped",
    "incoming circuit busy, active",
    "incoming circuit busy, locally blocked",
    "incoming circuit busy, remotely blocked",
    "incoming circuit busy, locally and remotely blocked",
    "outgoing circuit busy, active",
    "outgoing circuit busy, locally blocked",
    "outgoing circuit busy, remotely blocked",
    "outgoing circuit busy, locally and remotely blocked",
    "idle",
    "idle, active",
    "idle, locally blocked",
    "idle, remotely blocked",
    "idle, locally and remotely blocked"
};


/* Define Instance and Entity IDs required of any application that
 * utilized the ISUP API.
 */

#define ENT_ID     0x1a
#define INST_ID    0       /* Always 0 */

/* This application always uses isup sap 0 */

#define SAP_ID      0



/* internal function prototypes */
void handleStatInd(CTAHD ctahd, SiStaEvnt *event, IsupRcvInfoBlk *infoBlk);
void handleConStaInd(CTAHD ctahd, SiCnStEvnt *event, IsupRcvInfoBlk *infoBlk);

void usage(char *pname, int opt)
{
    fprintf(stderr, "opt - %d\n"
		    "Usage: %s [options]\n"
		    "\n"
		    "options:\n"
		    "       -i              switch type ITU\n"
		    "       -b number       specify board, default 1\n"
		    "       -c number       circuit to test with, default 1\n"
		    "\n", opt, pname);
    exit(128);
}

/* handleIsupMessage
 *
 * This function is called to handle all incoming ISUP messages.
 * It just calls the proper message handling function based on the
 * type of message received.
 */
DWORD handleIsupMessage(CTAHD ctahd)
{
    DWORD status;
    SiAllSdus       rcvEvent;
    IsupRcvInfoBlk  rcvInfo;
        U8      called[21];
        U8      cdLen;
        U8      calling[21];
        U8      cgLen;


    status = ISUPRetrieveMessage(ctahd, &rcvEvent, &rcvInfo, 1);
    if (status == ISUP_NOMSG)
    {
      fprintf(stderr, "ISUPRetrieveMessage() did not get a message, probably a congestion event \n");
      return(ISUP_SUCCESS);
    }
    if (status != ISUP_SUCCESS)
    {
      fprintf(stderr, "ISUPRetrieveMessage() failed, returned %d\n", status);
      return(status);
    }
	/* determine indication/confirmation type received */
    switch(rcvInfo.indType)
    {
    case EVTSITBNDCFM:              /* bind confirmation */
      printf("term: Bind Confirmation \n");
	    break;

    case EVTSITCONCFM:              /* connect confirmation */
	    printf("term: Unexpected Connect Confirmation for circuit %ld\n", rcvInfo.circuit);
	    break;

    case EVTSITCONIND:              /* connect indication (incoming call) */
	    printf("term: Connect Indication received for circuit %ld, ", rcvInfo.circuit);
      if (rcvEvent.m.siConEvnt.cgPtyNum.addrSig.pres == PRESENT)
      {
        /* convert BCD encoded address signals to ASCII for display */
        cdLen = ((rcvEvent.m.siConEvnt.cdPtyNum.addrSig.len * (U8)2) -
          (U8)((rcvEvent.m.siConEvnt.cdPtyNum.oddEven.val == NMB_EVEN) ? 0 : 1));
        ISUPBCDtoASCII(rcvEvent.m.siConEvnt.cdPtyNum.addrSig.val, cdLen, called, 21);

        cgLen = ((rcvEvent.m.siConEvnt.cgPtyNum.addrSig.len * (U8)2) -
          (U8)((rcvEvent.m.siConEvnt.cgPtyNum.oddEven.val == NMB_EVEN) ? 0 : 1));
        ISUPBCDtoASCII(rcvEvent.m.siConEvnt.cgPtyNum.addrSig.val, cgLen, calling, 21);

        printf("term: cd = %s, cg = %s\n", called, calling);

        /* send address complete message (ACM) */
        ISUPInitACM(switchType, &rcvEvent.m.siCnStEvnt);
            status = ISUPConnectStatusReq(ctahd, rcvInfo.suId, rcvInfo.suInstId, rcvInfo.spInstId,
                  rcvInfo.circuit, &rcvEvent.m.siCnStEvnt, ADDRCMPLT);
        if (status != ISUP_SUCCESS)
            printf("term: ISUPReleaseResp() failed status = %d\n", status);
        else
            printf("term: Address Complete sent for circuit %ld\n", rcvInfo.circuit);

        /* send answer message (ANM) */
        ISUPInitANM(switchType, &rcvEvent.m.siConEvnt);
            status = ISUPConnectResp(ctahd, rcvInfo.suId, rcvInfo.suInstId, rcvInfo.spInstId,
                  rcvInfo.circuit, &rcvEvent.m.siConEvnt);
        if (status != ISUP_SUCCESS)
            printf("term: ISUPConnectResp() failed status = %d\n", status);
        else
            printf("term: Answer sent for circuit %ld\n", rcvInfo.circuit);
        }
        else
        {
          /* send information request (INR) to acquire calling party address */
          ISUPInitINR(switchType, &rcvEvent.m.siCnStEvnt);
          rcvEvent.m.siCnStEvnt.infoReqInd.cgPtyAdReqInd.pres   = PRESENT;
          rcvEvent.m.siCnStEvnt.infoReqInd.cgPtyAdReqInd.val    = CGPRTYADDREQ_REQ;
              status = ISUPConnectStatusReq(ctahd, rcvInfo.suId, rcvInfo.suInstId, rcvInfo.spInstId,
              rcvInfo.circuit, &rcvEvent.m.siCnStEvnt, INFORMATREQ);
          if (status != ISUP_SUCCESS)
              printf("term: ISUPConnectStatusReq() failed status = %d\n", status);
          else
              printf("term: Information Request sent for circuit %ld\n", rcvInfo.circuit);

        }
	    break;

    case EVTSITDATIND:              /* data indication (user-to-user data) */
	    printf("term: Unexpected Data Indication for circuit %ld\n", rcvInfo.circuit);
	    break;

    case EVTSITSUSPIND:             /* suspend indication */
	    printf("term: Unexpected Suspend Indication for circuit %ld\n", rcvInfo.circuit);
	    break;

    case EVTSITRESMIND:             /* resume indication */
	    printf("term: Unexpected Resume Indication for circuit %ld\n",
		    (int) rcvInfo.circuit);
	    break;

    case EVTSITCNSTIND:             /* connect status indication */
      handleConStaInd(ctahd, &rcvEvent.m.siCnStEvnt, &rcvInfo);
	    break;

    case EVTSITRELIND:              /* Release indication */
	    printf("term: Release Indication received for circuit %ld\n", rcvInfo.circuit);

      status = ISUPReleaseResp(ctahd, rcvInfo.suId, rcvInfo.suInstId,
				    rcvInfo.spInstId, rcvInfo.circuit, &rcvEvent.m.siRelEvnt);
	    if (status != ISUP_SUCCESS)
        printf("term: ISUPReleaseResp() failed status = %d\n", status);
	    else
        printf("term: Release Complete sent for circuit %ld\n", rcvInfo.circuit);
	    break;

    case EVTSITRELCFM:              /* Release confirmation (release complt) */
	    printf("term: Unexpected Release Confirmation for circuit %ld\n", rcvInfo.circuit);
	    break;

    case EVTSITFACIND:              /* Facility indication */
	    printf("term: Unexpected Facility Indication for circuit %ld\n", rcvInfo.circuit);
	    break;

    case EVTSITSTAIND:              /* status indication */
      handleStatInd(ctahd, &rcvEvent.m.siStaEvnt, &rcvInfo);
	    break;

    default:
	    /* unrecognized event */
	    printf("term: Unrecognized indication [%d] for circuit %ld\n", (int) rcvInfo.indType, rcvInfo.circuit);
	    break;

    }

    return(ISUP_SUCCESS);

}       /* end handleIsupMessage */

/* handleStatInd
*
* This function handles a status indication from the far exchange
* or ISUP layer. A ERRORIND indicates that the call has failed, so
* the application terminates. Other indications are just logged (a real
* application would need to respond to some of these indications, such
* as circuit/group blocking/unblocking requests).
*/
void handleStatInd(CTAHD ctahd, SiStaEvnt *event, IsupRcvInfoBlk *infoBlk)
{

	switch (infoBlk->evntType)
	{

    case ERRORIND:   /* ERROR indication from ISUP layer */
      if (event->causeDgn.eh.pres     && event->causeDgn.causeVal.pres)
      {
        printf("term: ERROR Status, circ=%d, suId=%d, spId=%d, cause=%d\n",
            infoBlk->circuit, (int) infoBlk->suInstId, (int) infoBlk->spInstId,
          (int) event->causeDgn.causeVal.val);
      }
      else
      {
        printf("term: ERROR Status, circ=%d, suId=%d, spId=%d, no cause value\n",
          infoBlk->circuit, (int) infoBlk->suInstId, (int) infoBlk->spInstId);
      }
      break;

    case CONTCHK:           /* continuity check requested */
      printf("term: Unexpected Continuity Check Rqst Rcvd, circ=%d, suId=%d, spId=%d\n",
          infoBlk->circuit, (int) infoBlk->suInstId, (int) infoBlk->spInstId);
      break;

    case CONTREP:
      printf("term: Unexpected Continuity Response Rcvd, circ=%d, suId=%d, spId=%d\n",
        infoBlk->circuit, (int) infoBlk->suInstId, (int) infoBlk->spInstId);
      break;

    case CIRGRPQRYRSP:
      {
	    int  i;

	    printf("term: Circuit Query Response, circuit = %d, range = %d\n",
		    infoBlk->circuit, event->rangStat.range.val);

	    for (i = 0; i <= event->rangStat.range.val; i++)
        printf("term: circuit %ld state: %s\n", (infoBlk->circuit + i),
			    circuitStates[event->cirStateInd.cirSteInd.val[i]]);
	    break;
      }

    case CIRBLKREQ:
      {
        DWORD       status;

        status = ISUPStatusReq(ctahd, infoBlk->suId, infoBlk->suInstId,
            infoBlk->spInstId, 0, infoBlk->circuit, CIRBLKRSP, event);
        if(status != ISUP_SUCCESS)
        {
            printf("term: ISUPStatusReq() failed sending Block response, status = %d\n", status);
        }
        else
        {
            printf("term: Block response sent for circuit %ld\n", infoBlk->circuit);
        }
        break;
      }
    case CIRBLKRSP:
      printf("term: Circuit Block Response, circuit %ld\n", infoBlk->circuit);
      break;

    case CIRUNBLKREQ:
    {
      DWORD       status;

      status = ISUPStatusReq(ctahd, infoBlk->suId, infoBlk->suInstId,
              infoBlk->spInstId, 0, infoBlk->circuit, CIRUNBLKRSP, event);
      if(status != ISUP_SUCCESS)
      {
          printf("term: ISUPStatusReq() failed sending Unblock response, status = %d\n", status);
      }
      else
      {
          printf("term: Unblock response sent for circuit %ld\n", infoBlk->circuit);
      }
      break;
    }

    case CIRUNBLKRSP:
      printf("term: Circuit Unblock Response, circuit %ld\n", infoBlk->circuit);
      break;

    case CIRGRPBLKREQ:
    {
      DWORD       status;

      status = ISUPStatusReq(ctahd, infoBlk->suId, infoBlk->suInstId,
			    infoBlk->spInstId, 0, infoBlk->circuit, CIRGRPBLKRSP, event);
      if(status != ISUP_SUCCESS)
      {
          printf("term: ISUPStatusReq() failed sending Group Block response, status = %d\n", status);
      }
      else
      {
          printf("term: Group Block response sent for circuit %ld\n", infoBlk->circuit);
      }
      break;
    }

    case CIRGRPBLKRSP:
      printf("term: Circuit Group Block Response, starting at circuit %ld\n", infoBlk->circuit);
      break;

    case CIRGRPUNBLKREQ:
    {
      DWORD       status;

      status = ISUPStatusReq(ctahd, infoBlk->suId, infoBlk->suInstId,
			    infoBlk->spInstId, 0, infoBlk->circuit, CIRGRPUNBLKRSP, event);
      if(status != ISUP_SUCCESS)
      {
          printf("term: ISUPStatusReq() failed sending Group Unblock Response, status = %d\n", status);
      }
      else
      {
          printf("term: Group Unblock response sent for circuit %ld\n", infoBlk->circuit);
      }
      break;
    }

    case CIRGRPUNBLKRSP:
      printf("term: Circuit Group Unblock Response, starting at circuit %ld\n", infoBlk->circuit);
      break;

	case CIRRESERVE:
      printf("term: Unexpected Circuit Reservation circuit %ld.\n", infoBlk->circuit);
      break;

    case CIRRESREQ:
      printf("term: Circuit Reset circuit %ld.\n", infoBlk->circuit);
      break;

    case CIRGRPRESREQ:
      printf("term: Unexpected Circuit Group Reset, starting at circuit %ld.\n", infoBlk->circuit);
      break;

    case MTPPAUSE:
      printf("term: mtp pause\n");
      break;

    case MTPRESUME:
      printf("term: mtp resume\n");
      break;

    case MTPBACKUP:
      printf("term: mtp backup\n");
      break;

    case MTPPRIMARY:
      printf("term: mtp primary\n");
      break;

    case MTPSTANDALONE:
      printf("term: mtp stand alone\n");
      break;

	default:        /* all other status indications */
    printf("term: Unexpected Status received, circ=%d, suInstId=%d, spInstId=%d, type=%d\n",
          infoBlk->circuit, (int) infoBlk->suInstId, (int) infoBlk->spInstId, (int) infoBlk->evntType);
		break;

   }    /* switch */

} /* end of handleStatInd */

/* handleConStaInd
*
* This function handles a connection status indication event; if this
* is a info request for calling party number, an INFormation message is
* generated containing calling party number (only).
*
*/
void handleConStaInd(CTAHD ctahd, SiCnStEvnt *event, IsupRcvInfoBlk *infoBlk)
{
  U8 i;
  switch (infoBlk->evntType)
  {

	case ADDRCMPLT:
    printf("term: Address Complete received for circuit %ld\n", infoBlk->circuit);
		break;

	case MODIFY:                                                    /* modify request */
    printf("term: Modify request received for circuit %ld\n", infoBlk->circuit);
    break;

	case MODCMPLT:                                                  /* modify complete */
    printf("term: Modify Complete indication received for circuit %ld\n", infoBlk->circuit);
    break;

	case MODREJ:                                                    /* modify rejected */
    printf("term: Modify Reject indication received for circuit %ld\n", infoBlk->circuit);
    break;

	case PROGRESS:                                                  /* call progress info */
		/* assumes mandatory event info is present */
    printf("term: Progress (%d) indication received for circuit %ld\n",
      (int) event->evntInfo.evntInd.val, infoBlk->circuit);
    break;

	case FRWDTRSFR:                                                 /* forward transfer */
    printf("term: Forward Transfer indication received for circuit %ld\n", infoBlk->circuit);
    break;

	case INFORMATION:                                               /* information indication */
    {
    SiAllSdus   sendEvent;
    U8          calling[21];
    U8          cgLen;
    DWORD       status;

    printf("term: Information indication received for circuit %ld\n", infoBlk->circuit);
    /* convert BCD encoded address signals to ASCII for display */
    cgLen = ((event->cgPtyNum.addrSig.len * (U8)2) -
      (U8)((event->cgPtyNum.oddEven.val == NMB_EVEN) ? 0 : 1));
    ISUPBCDtoASCII(event->cgPtyNum.addrSig.val, cgLen, calling, 21);

    printf("term: Calling party number: %s\n", calling);
    /* send address complete message (ACM) */
    ISUPInitACM(switchType, &sendEvent.m.siCnStEvnt);
    status = ISUPConnectStatusReq(ctahd, infoBlk->suId, infoBlk->suInstId, infoBlk->spInstId,
						infoBlk->circuit, &sendEvent.m.siCnStEvnt, ADDRCMPLT);
    if (status != ISUP_SUCCESS)
        printf("term: ISUPReleaseResp() failed status = %d\n", status);
    else
        printf("term: Address Complete sent for circuit %ld\n", infoBlk->circuit);

    /* send answer message (ANM) */
    ISUPInitANM(switchType, &sendEvent.m.siConEvnt);
    status = ISUPConnectResp(ctahd, infoBlk->suId, infoBlk->suInstId, infoBlk->spInstId,
					    infoBlk->circuit, &sendEvent.m.siConEvnt);
    if (status != ISUP_SUCCESS)
        printf("term: ISUPConnectResp() failed status = %d\n", status);
    else
        printf("term: Answer sent for circuit %ld\n", infoBlk->circuit);

    break;
    }

	case INFORMATREQ:                                               /* information rqst */
    printf("term: Information Rqst received for circuit %ld\n", infoBlk->circuit);
    break;

	case SUBSADDR:                                                  /* subsequent address */
    printf("term: Subsequent Address received for circuit %ld\n", infoBlk->circuit);
    break;

  case SEGMENTATION:                                                  /* Segmentation address */
    printf("term: Segmentation received for circuit %ld\n", infoBlk->circuit);
    printf("Acc Transport in SGM %d: ", event->accTrnspt.eh.pres);
    for(i=0; i<event->accTrnspt.infoElmts.len; ++i)
      printf("%x ", event->accTrnspt.infoElmts.val[i]);
    printf("\n");
    break;


	default:
		/* unrecognized connect status indication */
    printf("term: Unrecognized connect status ind., circ=%d, suId=%d, spId=%d\n",
				(int) infoBlk->circuit, (int) infoBlk->suInstId,
		(int) infoBlk->spInstId);
		break;

	}       /* of switch */

} /* end of handleCnStInd */


static void ErrorAndExit( CTAHD Ctahd, DWORD ErrId, char *Fcn )
{

   printf( "Error : (%d) error returned in (%s) call.\n", ErrId, Fcn );
   exit( 1 );
}

void DemoShowEvent( CTA_EVENT *event )
{
    char format_buffer[CTA_MAX_FORMAT_SIZE];
    char *prefix = "\t\t\t";  /* default demo indent */
    format_buffer[0] = '\0';
    ctaFormatEvent( prefix, event, format_buffer, CTA_MAX_FORMAT_SIZE );
    printf( "%s", format_buffer );
}

/* main()
 *
 * This is the main routine for the ISUP incoming test application.
 * It initializes the application's local circuit state database,
 * initializes the ISUP API (by binding to ISUP), and then just loops
 * waiting for messages from ISUP and dispatching the appropriate
 * routine to handle the message.
 */

int main(int argc, char *argv[])
{
    short   optind = 1;
    char   *str;
    CTA_SERVICE_DESC isupOpenSvcLst[] = {{{"ISUP", "ISUPMGR"}, {0}, {0}, {0}}};
    char *isupCloseSvcLst[] = { "ISUP" };

    CTAQUEUEHD      FstQueue = 0;
    CTAHD           FstCtaHd = NULL_CTAHD;
    CTA_EVENT         Event;
    DWORD           Ret;
    CTA_INIT_PARMS      isupInitparms =                 { 0 };
    CTA_SERVICE_NAME    isupServiceNames[] =            { { "ISUP", "ISUPMGR" } };
    unsigned   DaemonRunning = 1;
    U8         inloop = 1;
    char        c;
    DWORD       status;
    ISUPAPISTATS pStats;

    while (str = argv[optind])
    {
      if (*str == '-')
      {
        optind++;
        switch (*++str)
        {
          case 'b':   /* which channel should we speak through */
            if (*++str)         /* was there a space ? */
              boardNum = atoi(str);
            else
            {
              if (argv[optind] == NULL)
                goto usage_and_exit;
              boardNum = atoi(argv[optind++]);
            }
            break;

          case 'i':   /* ITU ? */
            switchType = ST_ITU;
            break;

          case 's':   /* BICC ? */
            switchType = ST_BICC;
            break;

          default:
	usage_and_exit:
            printf("term: Usage: %s [-b <board>] \n", argv[0]);
            exit(1);
        }
      }
    }
   isupInitparms.size = sizeof(CTA_INIT_PARMS);
   /*isupInitparms.traceflags = CTA_TRACE_ENABLE; */
   /*isupInitparms.parmflags = CTA_PARM_MGMT_SHARED; */
   isupInitparms.traceflags = 0;
   isupInitparms.parmflags = 0;
   isupInitparms.ctacompatlevel = CTA_COMPATLEVEL;

   Ret = ctaInitialize(isupServiceNames, 1, &isupInitparms);
   if (Ret != SUCCESS)
   {
           ErrorAndExit( NULL_CTAHD, Ret, "ctaInitialize" );
   }


   Ret = ctaCreateQueue( NULL, 0, &FstQueue );
   if ( Ret != SUCCESS )
   {
           ErrorAndExit( NULL_CTAHD, Ret, "ctaCreateQueue" );
   }

   /* Open FstCtaHd on FstQueue                   */
   Ret = ctaCreateContext( FstQueue, 0, NULL, &FstCtaHd );
   if ( Ret != SUCCESS )
   {
           ErrorAndExit( NULL_CTAHD, Ret, "ctaCreateContext" );
   }

   isupOpenSvcLst[0].svcargs.args[0] = boardNum;     /* board number */
   isupOpenSvcLst[0].svcargs.args[1] = INST_ID;      /* srcInst */
   isupOpenSvcLst[0].svcargs.args[2] = ENT_ID;       /* dprChan */
   isupOpenSvcLst[0].svcargs.args[3] = 1;            /* AutoBind? (yes=1, no= 0) */
   isupOpenSvcLst[0].svcargs.args[4] = SAP_ID;       /* spId */
   isupOpenSvcLst[0].svcargs.args[5] = SAP_ID;       /* suId */

   Ret = ctaOpenServices( FstCtaHd, isupOpenSvcLst, 1 );
   if ( Ret != SUCCESS )
   {
           ErrorAndExit( FstCtaHd, Ret, "ctaOpenServices" );
   }

   /* Wait for service open to complete. */
   Ret = ctaWaitEvent( FstQueue, &Event, CTA_WAIT_FOREVER );
   /*DemoShowEvent(&Event);*/
   if ( Ret != SUCCESS )
   {
           ErrorAndExit( FstCtaHd, Ret, "ctaWaitEvent" );
   }

   if (( Event.id != CTAEVN_OPEN_SERVICES_DONE )  || ( Event.value != CTA_REASON_FINISHED ))
   {
           ErrorAndExit( FstCtaHd, Ret, "ctaWaitEvent" );
   }

    while ( inloop)
    {
        /* cta wait event */
        Ret = ctaWaitEvent( FstQueue, &Event, 5000 );
        /*DemoShowEvent(&Event);*/
        if ( Ret != SUCCESS )
        {
                ErrorAndExit( FstCtaHd, Ret, "ctaWaitEvent" );
        }
        switch ( Event.id )
        {
            case ISUPEVN_DATA:
            /* an ISUP event has occured, call ISUPRetrieveMessage() to get the event */
            handleIsupMessage(FstCtaHd);
            break;

            case CTAEVN_WAIT_TIMEOUT:
            /* 5 seconds passed, check for keyboard input */
                /* make sure a key was hit */
                if (_kbhit())
                {
                  c = _getch();
                  if (c == 's')
                  {
                      status = ISUPGetApiStats(FstCtaHd, &pStats, 0);
                      if (status != ISUP_SUCCESS)
                        printf("orig: ISUPGetApiStats() failed status = %d\n", status);
                      else
                      {
                        printf("qCount = %x\n", pStats.qCount);
                        printf("qPeak = %x\n", pStats.qPeak);
                        printf("txPending = %x\n", pStats.txPending);
                        printf("txPendPeak = %x\n", pStats.txPendPeak);
                        printf("txSuccess = %x\n", pStats.txSuccess);
                        printf("txFailed = %x\n", pStats.txFailed);
                        printf("txLastErr = %x\n", pStats.txLastErr);
                        printf("rxSuccess = %x\n", pStats.rxSuccess);
                        printf("apiQCongLvl = %x\n", pStats.apiQCongLvl);
                        printf("isupCongLvl = %x\n", pStats.isupCongLvl);
                        printf("isupCongSrc = %x\n", pStats.isupCongSrc);
                      }
                  }
                  else inloop = 0;
                }
            break;

            default:
                printf( "CTAHD(0x%08X) : Unhandled event id (0x%x) during "
                        "wait.\nExiting...\n", Event.ctahd,
                        Event.id );
        }


    }           /* end while */

   /*
   *   Close service and destroy context.
   */
   Ret = ctaCloseServices( FstCtaHd, isupCloseSvcLst, 1 );
   if ( Ret != SUCCESS )
   {
      ErrorAndExit( FstCtaHd, Ret, "ctaCloseServices" );
   }

   /* Wait for service close to complete. */
   Ret = ctaWaitEvent( FstQueue, &Event, CTA_WAIT_FOREVER );
   if ( Ret != SUCCESS )
   {
           ErrorAndExit( FstCtaHd, Ret, "ctaWaitEvent" );
   }
   if (( Event.id != CTAEVN_CLOSE_SERVICES_DONE )  || ( Event.value != CTA_REASON_FINISHED ))
   {
           ErrorAndExit( FstCtaHd, Ret, "ctaWaitEvent" );
   }

   Ret = ctaDestroyContext( FstCtaHd );

   if ( Ret != SUCCESS )
   {
      ErrorAndExit( FstCtaHd, Ret, "ctaDestroyContext" );
   }

   /* Wait for context close to complete. */
   Ret = ctaWaitEvent( FstQueue, &Event, CTA_WAIT_FOREVER );
   if ( Ret != SUCCESS )
   {
           ErrorAndExit( FstCtaHd, Ret, "ctaWaitEvent" );
   }
   if (( Event.id != CTAEVN_DESTROY_CONTEXT_DONE )  || ( Event.value != CTA_REASON_FINISHED ))
   {
           ErrorAndExit( FstCtaHd, Ret, "ctaWaitEvent" );
   }

   Ret = ctaDestroyQueue( FstQueue );
   if ( Ret != SUCCESS )
   {
      ErrorAndExit( NULL_CTAHD, Ret, "ctaDestroyQueue" );
   }

} /* end of main */

#ifdef UNIX
/******************************************************************************
 * Function:   _kbhit
 * Returns:
 * Notes:      Implements WinNT-like _kbhit
 */
int _kbhit(void)
{
   int RetCode = 0;
   struct pollfd kbdfd;

   kbdfd.fd = 0; /*input*/
   kbdfd.events = POLLIN;
   kbdfd.revents = 0;

   RetCode = poll(&kbdfd,1,100);
   if (RetCode <= 0) {
      RetCode = 0; /*timeout*/
   } else {
      RetCode = 1;
   }

   return (RetCode);
}
#endif

